<li class="{{#listview}}list-dir-item-listview{{/listview}} list-dir-item cm-media-list-item">
    {{#maychangecoverart}}
    <span class="cm-media-list-wrench glyphicon glyphicon-wrench" data-dirname="{{directoryname}}"></span>
    {{/maychangecoverart}}
    <a dir="{{dirpath}}" href="javascript:;" class="list-dir" title="{{dirpath}}">
        {{^isrootdir}}
            <div class="list-dir-albumart unloaded" search-data="{{coverarturl}}">
                <img src="res/img/folder.png" />
            </div>
        {{/isrootdir}}
        <div class="list-dir-name-wrap">
            <span class="list-dir-name">{{label}}</span>
        </div>
    </a>
</li>
